/*
 * simpledraw
 * Copyright (C) 2004 Kohta NAKASHIMA
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>

#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "data_struct.h"


fig_list_t *fig_list_head = NULL;
fig_list_t *fig_list_tail = NULL;
enum figtype mode = LINE;

unsigned drawing_flag = 0;
enum filemode filemode = OPEN;
int fig_count = 0;

void clear_figures(){
    fig_list_t *cur,*next;
    
    for(cur = fig_list_head; cur; cur = next){
        next = cur->next;
        free(cur);
    }
    fig_list_head = NULL;
    fig_list_tail = NULL;
}

void load_figures(const gchar *filename){
    FILE *fp;
    gchar sig[4];
    fig_list_t *newitem;
    int count,i;

    if((fp = fopen(filename,"r"))==NULL){
        g_print("Cannot open %s\n",filename);
        return;
    }

    fread(sig, sizeof(gchar), 4, fp);
    if(memcmp(sig,"SDRW",4)){
        g_print("%s is not simpledraw file\n",filename);
        return;
    }
    clear_figures();

    fread(&count, sizeof(int), 1, fp);

    for(i=0;i<count;i++){
        newitem = malloc(sizeof(fig_list_t));
        fread(newitem,sizeof(fig_list_t), 1, fp);
        newitem->next = NULL;
        fig_count++;
        if(fig_list_head == NULL){
            fig_list_head = newitem;
        }else{
            fig_list_tail->next = newitem;
        }
        fig_list_tail = newitem;
    }
    fclose(fp);
    return;
}

void save_figures(const gchar *filename){
    FILE *fp;
    fig_list_t *cur;

    if((fp = fopen(filename,"w"))==NULL){
        g_print("Cannot open %s\n",filename);
        return;
    }

    fwrite("SDRW", sizeof(gchar), 4, fp);
    fwrite(&fig_count, sizeof(int), 1, fp);

    for(cur = fig_list_head; cur; cur = cur->next){
        fwrite(cur,sizeof(fig_list_t), 1, fp);
    }
    fclose(fp);
    return;
}


void
on_clear1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *drawingarea;

    clear_figures();

    drawingarea = lookup_widget((GtkWidget *)menuitem,"drawingarea1");
    printf("darea = %p\n",drawingarea);
    gtk_widget_queue_draw(drawingarea);
}

void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *filesel;

    filemode = OPEN;
    filesel = create_fileselection1();
    gtk_widget_show(filesel);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *filesel;

    filemode = SAVE;
    filesel = create_fileselection1();
    gtk_widget_show(filesel);
}

/* $B%a%K%e!<!V(Bquit$B!W$KBP$9$k%7%0%J%k%O%s%I%i(B */
void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


void
on_line1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    mode = LINE;
}


void
on_rectangle1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    mode = RECT;
}


void
on_ellipse1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    mode = ELLI;
}

/* $B%a%K%e!<!V(BAbout$B!W$KBP$9$k%7%0%J%k%O%s%I%i(B */
void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *about;

    about = create_dialog1();
    gtk_widget_show(about);

}

/* $BJD$8$k%\%?%s$,2!$5$l!"%&%$%s%I%&$,JD$8$i$l$k;~$N%7%0%J%k%O%s%I%i(B */
gboolean
on_window1_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

    return FALSE;
}

/* delete_event$B$N%O%s%I%i$,(BFALSE$B$rJV$9$HAw$i$l$k(Bdestry$B%7%0%J%k$N%O%s%I%i!#(B */
void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
    fig_list_t *cur;
    gint x, y, width, height;

    for(cur = fig_list_head; cur ; cur = cur->next){
        switch(cur->type) {
        case LINE:
            gdk_draw_line(widget->window,widget->style->fg_gc[widget->state],
                          cur->fig.line.sp.x, cur->fig.line.sp.y,
                          cur->fig.line.ep.x, cur->fig.line.ep.y);
            break;
        case RECT:
            x  = cur->fig.rect.x;
            y  = cur->fig.rect.y;
            width  = cur->fig.rect.width;
            height = cur->fig.rect.height;
            if(width < 0) {
                x = x + width;
                width = -width;
            }
            if(height < 0) {
                y = y + height;
                height = -height;
            }
            gdk_draw_rectangle(widget->window,widget->style->fg_gc[widget->state], 
                               FALSE, /* not filled */
                               x,y,width,height);
            break;
        case ELLI:
            x  = cur->fig.rect.x;
            y  = cur->fig.rect.y;
            width  = cur->fig.rect.width;
            height = cur->fig.rect.height;
            if(width < 0) {
                x = x + width;
                width = -width;
            }
            if(height < 0) {
                y = y + height;
                height = -height;
            }
            gdk_draw_arc(widget->window,widget->style->fg_gc[widget->state], 
                         FALSE, /* not filled */
                         x,y,width,height,
                         0, 360*64);
            break;
        default:

            break;
        }
    }

    return TRUE;
}

gboolean
on_drawingarea1_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
     gint cur_x,cur_y;
     fig_list_t *newitem;

     drawing_flag = 1;

     cur_x = (gint)event->x;
     cur_y = (gint)event->y;

     if((newitem = malloc(sizeof(fig_list_t)))==NULL){
         g_print("malloc failed\n");
         return FALSE;
     }

     switch(mode){
     case LINE:
         newitem->type = LINE;
         newitem->fig.line.sp.x = cur_x; newitem->fig.line.sp.y = cur_y;
         newitem->fig.line.ep.x = cur_x; newitem->fig.line.ep.y = cur_y;
         break;
     case RECT:
         newitem->type = RECT;
         newitem->fig.rect.x = cur_x; 
         newitem->fig.rect.y = cur_y;
         newitem->fig.rect.width = 0;
         newitem->fig.rect.height= 0;
         break;
     case ELLI:
         newitem->type = ELLI;
         newitem->fig.elli.x = cur_x; 
         newitem->fig.elli.y = cur_y;
         newitem->fig.elli.width = 0;
         newitem->fig.elli.height= 0;
         break;
     default:
         break;
     }
     newitem->next = NULL;
     fig_count++;
     if(fig_list_head == NULL){
         fig_list_head = newitem;
     }else{
         fig_list_tail->next = newitem;
     }
     fig_list_tail = newitem;

     return TRUE;
}

gboolean
on_drawingarea1_button_release_event   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    drawing_flag = 0; /* $B$3$3$rDI2C(B */

    /* $B=*E@$N@_Dj=hM}$O(B motion_notify_event$B$G9T$&$N$G:o=|(B  */

    return TRUE;
}


gboolean
on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
     gint cur_x,cur_y;

     if(drawing_flag){
         cur_x = (gint)event->x;
         cur_y = (gint)event->y;
         switch(fig_list_tail->type){
         case LINE:
             fig_list_tail->fig.line.ep.x = cur_x;
             fig_list_tail->fig.line.ep.y = cur_y;
             break;
         case RECT:
             fig_list_tail->fig.rect.width = cur_x - fig_list_tail->fig.rect.x;
             fig_list_tail->fig.rect.height= cur_y - fig_list_tail->fig.rect.y;
             break;
         case ELLI:
             fig_list_tail->fig.elli.width = cur_x - fig_list_tail->fig.elli.x;
             fig_list_tail->fig.elli.height= cur_y - fig_list_tail->fig.elli.y;
             break;
         default:
             break;
         }

         gtk_widget_queue_draw(widget);
     }

     return TRUE;
}


void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *filesel;
    const gchar *filename;


    filesel = lookup_widget((GtkWidget *)button, "fileselection1");

    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));

    switch(filemode){
    case OPEN:
        load_figures(filename);
        break;
    case SAVE:
        save_figures(filename);
        break;
    default:
        break;
    }

    gtk_widget_queue_draw(g_drawarea);

    gtk_widget_destroy(filesel);
}


void
on_cancel_button1_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *filesel;

    filesel = lookup_widget((GtkWidget *)button, "fileselection1");
    gtk_widget_destroy(filesel);
}



void
on_okbutton1_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *about;

    about = lookup_widget((GtkWidget *)button, "dialog1");
    gtk_widget_destroy(about);
}

